package com.keyware.composeanalysis.mongo;

import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.List;


/**
 * 此类映射到名为 "project_assembly" 的 MongoDB 集合。
 * <p>
 * 当前项目匹配到的开源项目的信息
 *
 * @author liuzongren
 * @date 2024/7/9
 */
@Document(collection = "project_assembly")
@Data
@Accessors(chain = true)
public class ProjectAssemblyMongoDto {

    /**
     * MongoDB 文档的唯一标识符。
     */
    @Id
    private String id;

    /**
     * 项目的文件数量
     */
    private Integer fileCount;

    /**
     * 匹配到的开源文件的数量
     */
    private Integer matchFileCount;

    /**
     * 匹配到的开源项目版本Id
     */
    private String versionId;

    /**
     * 匹配到的开源项目版本名称
     */
    private String versionName;

    /**
     * 匹配到的开源项目在开源网站的项目序号
     */
    private String projectId;

    /**
     * 匹配到的开源项目的名称。
     */
    private String projectName;

    /**
     * 开源项目的地址
     */
    private String openSourceUrl;

    /**
     * 被测件和当前项目的整体相似度
     */
    private Double semblance;

    /**
     * 当前开源项目的开源协议
     */
    private List<String> licenseType;

}