package com.keyware.composeanalysis.task;

import com.keyware.composeanalysis.constant.enums.AnalysisLevelEnum;
import com.keyware.composeanalysis.entity.AnalysisTask;
import com.keyware.composeanalysis.mongo.FileDataMongoDto;
import org.springframework.data.mongodb.core.MongoTemplate;

import java.util.concurrent.CountDownLatch;

/**
 * @author liuzongren
 * @date 2024/7/31
 * @description
 */
public class AnalysisTaskFactory {

    /**
     * 根据分析类型，创建具体的分析任务
     * @param analysisTask
     * @param analysisFile
     * @param mongoTemplate
     * @param countDownLatch  任务总数控制器
     * @return
     */

    public static IAnalysisTask createAnalysisTask(AnalysisTask analysisTask, FileDataMongoDto analysisFile, MongoTemplate mongoTemplate, CountDownLatch countDownLatch) {
        AnalysisLevelEnum analysisLevel = AnalysisLevelEnum.getAnalysisLevelEnum(analysisTask.getAnalysisLevel());
        switch (analysisLevel) {
            case FILE_LEVEL:
                return new FileAnalysisTask(analysisTask,analysisFile,mongoTemplate,countDownLatch);
            case FUNCTION_LEVEL:
                return new FunctionAnalysisTask(analysisTask,analysisFile,mongoTemplate,countDownLatch);
            case BLOCK_LEVEL:
                return new CodeBlockAnalysisTask(analysisTask,analysisFile,mongoTemplate,countDownLatch);
            case LINE_LEVEL:
                return new LineAnalysisTask(analysisTask,analysisFile,mongoTemplate,countDownLatch);
            default:
                break;
        }
        return null;
    }



}
