package com.keyware.composeanalysis.util;

import cn.hutool.core.lang.Pair;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ByteUtil;
import cn.hutool.core.util.StrUtil;
import io.micrometer.common.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

public class SimilarityUtil {



    private SimilarityUtil() {

    }

    public static void main(String[] args) {
        String s1="";
        String s2="";
        double similarity = getSimilarityMe(s1, s2);
        System.out.println(similarity);
    }

    /**
     * 获得两个文件的相似度
     * @param sentence1
     * @param sentence2
     * @return
     */
    public static double getSimilarityMe(String sentence1, String sentence2) {
        //被测件文件行
        List<String> sent1Words = getSplitWords(sentence1);
        if (sentence1.length()==0){
            return 0.00;
        }
        //溯源到文件行
        HashSet<String> sent2Words = getSplitWords1(sentence2);
        //匹配到的行数
        double count=0;
        for (String sent1Word : sent1Words) {
            if (sent2Words.contains(sent1Word)){
                count++;
            }
        }
        return count/sent1Words.size();
    }


    /**
     * 获取开源率和开源行号
     * @param analysisFile  被测件内容
     * @param openSourceFile  开源文件内容
     * @return
     */
//    public static Pair<Float, HashSet<Integer>> getOpenRateAndSaveRowNum(String analysisFile, String openSourceFile) {
//        if (StrUtil.hasBlank(analysisFile,openSourceFile)){
//            return new Pair<>(0.00f,new HashSet<>());
//        }
//        //匹配到的行号
//        HashSet<Integer> matchedRowsNum = new HashSet<>();
//
//        //被测件文件行
//        List<String> analysisFileLineInfo = getSplitWords(analysisFile);
//
//        //溯源到文件行
//        HashSet<String> openSourceFileLineInfo = getSplitWords1(openSourceFile);
//
//        for (int i = 0; i < analysisFileLineInfo.size(); i++) {
//            String sent1Word = analysisFileLineInfo.get(i);
//            if (openSourceFileLineInfo.contains(sent1Word)) {
//                matchedRowsNum.add(i);
//            }
//        }
//
//        //计算开源率
//        BigDecimal openRate = new BigDecimal(matchedRowsNum.size()).divide(new BigDecimal(analysisFileLineInfo.size()), 4, RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2);
//
//        return new Pair<>(openRate.floatValue(), matchedRowsNum);
//    }


//    public static Pair<Float, HashSet<Integer>> getOpenRateAndSaveRowNum(byte[] analysisFile, byte[] openSourceFile) {
//        if (ArrayUtil.hasNull(analysisFile,openSourceFile)){
//            return new Pair<>(0.00f,new HashSet<>());
//        }
//        //匹配到的行号
//        HashSet<Integer> matchedRowsNum = new HashSet<>();
//
//        //被测件文件行
//        List<String> analysisFileLineInfo = getSplitWords(new String(analysisFile));
//
//        //溯源到文件行
//        HashSet<String> openSourceFileLineInfo = getSplitWords1(new String(openSourceFile));
//
//        for (int i = 0; i < analysisFileLineInfo.size(); i++) {
//            String sent1Word = analysisFileLineInfo.get(i);
//            if (openSourceFileLineInfo.contains(sent1Word)) {
//                matchedRowsNum.add(i);
//            }
//        }
//
//        //计算开源率
//        BigDecimal openRate = new BigDecimal(matchedRowsNum.size()).divide(new BigDecimal(analysisFileLineInfo.size()), 4, RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2);
//
//        return new Pair<>(openRate.floatValue(), matchedRowsNum);
//    }

    public static Pair<Float, HashSet<Integer>> getOpenRateAndSaveRowNum(String analysisFile, String openSourceFile) {
        if (StrUtil.hasBlank(analysisFile,openSourceFile)){
            return new Pair<>(0.00f,new HashSet<>());
        }
        //匹配到的行号
        HashSet<Integer> matchedRowsNum = new HashSet<>();

        //被测件文件行
        List<String> analysisFileLineInfo = getSplitWords(analysisFile);

        //溯源到文件行
        HashSet<String> openSourceFileLineInfo = getSplitWords1(openSourceFile);

        for (int i = 0; i < analysisFileLineInfo.size(); i++) {
            String sent1Word = analysisFileLineInfo.get(i);
            if (openSourceFileLineInfo.contains(sent1Word)) {
                matchedRowsNum.add(i);
            }
        }

        //计算开源率
        BigDecimal openRate = new BigDecimal(matchedRowsNum.size()).divide(new BigDecimal(analysisFileLineInfo.size()), 4, RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2);

        return new Pair<>(openRate.floatValue(), matchedRowsNum);
    }


    /**
     * 获得两个文件的相似度,并将被匹配的行
     * @param matchLineInfos  被匹配的行信息
     * @param sentence2    开源文件内容
     * @return
     */
    public static double getSimilarityAndSaveRowNum(List<String> matchLineInfos, String sentence2,HashSet<Integer> matchRows) {
        if (CollectionUtils.isEmpty(matchLineInfos)){
            return 0.00d;
        }
        //溯源到文件行
        HashSet<String> sent2Words = getSplitWords1(sentence2);
        //匹配到的行数
        double count = 0d;
        for (int i = 0; i < matchLineInfos.size(); i++) {
            String lineContents = matchLineInfos.get(i);
            if (sent2Words.contains(lineContents)) {
                //保存匹配中的行序号
                matchRows.add(i);
                count++;
            }
        }
        return count / matchLineInfos.size();
    }



    public static List<String> getSplitWords(String sentence) {
        List<String> lineList = new ArrayList<String>();
        if (StringUtils.isBlank(sentence)){
            return lineList;
        }
        sentence = sentence.replaceAll("\n\r", "\n").replaceAll("\r\n", "\n").replaceAll("\r", "\n");
        List<String> list = Arrays.asList(sentence.split("\n"));
        for (String string : list) {
            if (string != null && !"".equals(string.trim())) {
                lineList.add(string.replaceAll(" ",""));
            }
        }
        return lineList;

//        // 去除掉html标签
//
//        sentence = Jsoup.parse(sentence.replace(" ","")).body().text();
//
//
//        // 标点符号会被单独分为一个Term，去除之
//
//        return HanLP.segment(sentence).stream().map(a -> a.word).filter(s -> !"`~!@#$^&*()=|{}':;',\\[\\].<>/?~！@#￥……&*（）——|{}【】‘；：”“'。，、？ ".contains(s)).collect(Collectors.toList());

    }
    private static HashSet<String> getSplitWords1(String sentence) {
        HashSet<String> set = new HashSet<>();
        sentence = sentence.replaceAll("\n\r", "\n").replaceAll("\r\n", "\n").replaceAll("\r", "\n");
        List<String> list = Arrays.asList(sentence.split("\n"));
        for (String string : list) {
            if (string != null && !"".equals(string.trim())) {
                set.add(string.replaceAll(" ",""));
            }
        }
        return set;
    }

}
